package io.tswf.minecraft.blockventure.core.transmutation.transmute

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.exception.ServerWorldCustomStoragesSaveException
import io.tswf.minecraft.blockventure.core.transmutation.expansion.IServerWorldCustomDataStorageAccess
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.AtHead
import io.tswf.transmutations.transmute.annotation.Self
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.server.world.ServerWorld
import net.minecraft.util.ProgressListener

@CompileStatic
class TransmutationServerWorld {

    @Transmute(
            targetClass = @ClassReference(ServerWorld),
            targetMethod = @MethodDescription(
                    name = "save",
                    parameterTypeClasses = [ ProgressListener, ClassReference.BOOLEAN, ClassReference.BOOLEAN ],
                    returnTypeClass = ClassReference.VOID
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtHead
    static void handleServerWorldSave(ProgressListener progressListener, boolean flush, boolean savingDisabled, @Self ServerWorld world) {
        try {
            def storageAccess = (IServerWorldCustomDataStorageAccess) world
            def storages = storageAccess.blockventureCustomStorages

            for (def storage : storages.values()) {
                storage.commit()
            }
        } catch (Throwable e) {
            if (e instanceof ServerWorldCustomStoragesSaveException.Bypass) {
                throw e
            }

            throw new ServerWorldCustomStoragesSaveException(world, e)
        }
    }
}
